% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proportional.odds.modelling.R
\name{po}
\alias{po}
\title{Treatment Effects on Odds Ratio of Lower Severity over Time}
\usage{
po(
  formula,
  data,
  subject,
  treatment,
  imputation = F,
  common.odds.ratio = T,
  piecewise.linear = T,
  intercept = T,
  knots = NULL,
  control.ngd = list(learning.rate = 0.1, max.iter = 1000, eps = 1e-06, messages = F),
  start.time = NULL,
  end.time = NULL,
  imputed.score = 7
)
}
\arguments{
\item{formula}{A formula object, with all of the outcome variables
on the left hand side of a \code{~} operator and the treatment and covariates 
on the right. The outcome variables must be specified through the \code{outcome()} function. 
See \code{?outcome} and \code{?ph} for further information.}

\item{data}{A data.frame object. The data.frame in which to interpret the
variable names in formula. Must contain the subject ID, the treatment arm, 
the examination time, the clinical status measured  
at each examination, and any covariates. The clinical status must contain 
at least two categories at each examination time.
See \code{?ph} for further information.}

\item{subject}{A character string. Name of the variable in data which 
identifies multiple rows from the same subject.}

\item{treatment}{A character string. Name of the variable in data that 
corresponds to treatment arm.}

\item{imputation}{Logical. Whether the input data is imputed by carrying the last observation forward.
The default value if FALSE.}

\item{common.odds.ratio}{Logical. Whether the common odds ratio of
lower severity over time for treatment versus placebo is estimated.
The default value is TRUE.}

\item{piecewise.linear}{Logical. Whether a piecewise log-linear odds ratio 
of lower severity over time is estimated
and whether daily odds ratios of lower severity are estimated by
separate proportional odds models.
The default value is TRUE.
See Details for further information.}

\item{intercept}{Logical. Whether intercept is included for the piecewise linear 
function while assuming a piecewise log-linear odds ratio over time.
The default value is TRUE.
See Details for further information.}

\item{knots}{An integer vector or NULL, The potential change points in days of the
piecewise log-linear odds ratio.
See Details for further information.
If NULL, the function will set change points at the start of every week.}

\item{control.ngd}{List, with hyperparameters for natual gradient ascent algorithm. 
See Details for further information.}

\item{start.time}{Non-negative integer or \code{NULL}. The first examination day included in the analysis. 
If NULL, the first examination day is day 1 for estimating the common odds ratio and day 0 for 
estimating the piecewise log-linear odds ratio.}

\item{end.time}{Non-negative integer or \code{NULL}. The last examination day included in the analysis. 
If NULL, the last examination day is day 28 for estimating the common odds ratio 
and the piecewise log-linear odds ratio.}

\item{imputed.score}{Integer. The clinical status used for imputation 
when there are no measurements for a subject.
The default value is 7.}
}
\value{
A list object with the following elements:

\item{common.odds.ratio}{A list object contains common odds ratio estimate of lower severity over time
    for treatment versus placebo and its 95\% confidence interval.}
\item{piecewise.linear}{A list object contains change points, coefficient estimates, 
    and covariance matrix estimates for piecewise linear model. 
    It also contains a plot showing the estimated piecewise log-linear odds ratios and daily odds ratios, with
    corresponding 95\% confidence intervals.}
\item{separate}{A data.frame object contains the estimated daily odds ratios of lower severity for
    treatment versus placebo, together with corresponding 95\% confidence intervals, under separate proportional odds models.}
}
\description{
Treatment effect on the clinical status is characterized by
the odds ratio of lower severity under a series of proportional odds models.
The overall treatment effect over a time window is captured by 
a common odds ratio of lower severity over time.
A piecewise log-linear odds ratio of lower severity over time
can be estimated.
Missing values can be imputed internally by carrying the
last observation forward.
}
\details{
The piecewise linear function for the log odds raio
is determined by the \code{intercept} and \code{knots}.
If the intercept is excluded from the piecewise linear function,
the model assumes no treatment effect on the odds ratio of lower severity 
at examination time zero. 
The knots are the change points in the piecewise linear function.
For example, if we assume the treatment effect on log odds ratio of lower severity
has change points at examination time 0, 8, 15, 21, 
the input argument should be \code{knots = c(0, 8, 15, 21)}.

We use natural gradient ascent algorithm to maximize the log likelihood function of
the proportional odds model. 
The natural gradient ascent algorithm has four hyperparameters
\itemize{
\item \code{learning.rate} determines the step size for natural gradient ascent algorithm. 
    Must be numeric.
    The default value is 0.1.
\item \code{max.iter} is the maximum number of iterations for natural gradient ascent algorithm. 
    Must be integer.
    The default value is 1000.
\item \code{eps} is the convergence threshold for natural gradient ascent algorithm. 
    Must be numeric.
    The default value is \code{1e-6}.
\item \code{messages} decides whether to print out the coefficient estimates 
    and log likelihood during optimization. 
    Must be logical.
    The default value is \code{FALSE}.
}
}
\examples{
data(acttData)
model <- outcome(examination_time, clinical_status) ~ treatment_arm + baseline_severity
po(formula = model,
   data = acttData,
   subject = 'subject_id',
   treatment = 'treatment_arm',
   imputation = T,
   knots = c(0, 8, 13, 17, 24),
   start.time = 1,
   end.time = 28)
}
\references{
Lin DY, Wang J, Gu Y, Zeng D (2022). 
  Evaluating Treatment Efficacy in Hospitalized Covid-19 Patients. Submitted.
}
